from datetime import datetime, timezone
from functools import reduce
from typing import Any, Dict, Iterable, List, Optional

import pandas as pd
from fugue import (
    FugueWorkflow,
    PartitionSpec,
    Schema,
    make_execution_engine,
    transform,
    transformer,
)
from fugue.dataframe.utils import (
    normalize_dataframe_column_names,
    rename_dataframe_column_names,
)

import whylogs as why
from whylogs.core.view.column_profile_view import ColumnProfileView
from whylogs.core.view.dataset_profile_view import DatasetProfileView

_COL_NAME_FIELD = "__whylogs_fugue_col_name"
_COL_PROFILE_FIELD = "__whylogs_fugue_col_profile"
DF_PROFILE_FIELD = "__whylogs_df_profile_view"


def fugue_profile(
    df: Any,
    dataset_timestamp: Optional[datetime] = None,
    creation_timestamp: Optional[datetime] = None,
    partition: Any = None,
    profile_cols: Optional[List[str]] = None,
    as_local: bool = True,
    profile_field: str = DF_PROFILE_FIELD,
    engine: Any = None,
    engine_conf: Any = None,
    **kwargs,
) -> Any:
    if not isinstance(df, str):
        df, renames = normalize_dataframe_column_names(df)
    else:
        renames = {}
    profiler = _FugueProfiler(
        partition,
        profile_cols,
        renames=renames,
        dataset_timestamp=dataset_timestamp,
        creation_timestamp=creation_timestamp,
        profile_field=profile_field,
    )
    if len(profiler._by) == 0:
        assert len(kwargs) == 0, ValueError("Profiling without partition can not take extra arguments")
        return profiler.transform_no_logical_partition(df, engine=engine, engine_conf=engine_conf)
    return profiler.transform_with_logical_partition(
        df, engine=engine, engine_conf=engine_conf, as_local=as_local, **kwargs
    )


class _FugueProfiler:
    def __init__(
        self,
        partition,
        cols,
        renames: Dict[str, Any],
        dataset_timestamp: Optional[datetime] = None,
        creation_timestamp: Optional[datetime] = None,
        profile_field: str = DF_PROFILE_FIELD,
    ):
        now = datetime.now(timezone.utc)

        self._dataset_timestamp = dataset_timestamp or now
        self._creation_timestamp = creation_timestamp or now
        self._cols_to_orig = renames
        self._orig_to_cols = {v: k for k, v in renames.items()}

        part = PartitionSpec(partition)
        self._by = [self._orig_to_cols.get(x, x) for x in part.partition_by]
        self._partition = PartitionSpec(part, by=self._by)
        self._cols = cols
        self._profile_field = profile_field
        self._profile_schema = Schema([(profile_field, bytes)])

    def to_col_profiles(self, df: pd.DataFrame) -> Iterable[Dict[str, Any]]:
        if len(self._cols_to_orig) > 0:
            df = df.rename(columns=self._cols_to_orig)
        res = why.log(df[self._cols] if self._cols is not None else df)
        for col_name, col_profile in res.view().get_columns().items():
            yield {_COL_NAME_FIELD: col_name, _COL_PROFILE_FIELD: col_profile.serialize()}

    def merge_col_profiles(self, df: pd.DataFrame) -> pd.DataFrame:
        merged_profile: ColumnProfileView = reduce(
            lambda acc, x: acc.merge(x), df[_COL_PROFILE_FIELD].apply(lambda x: ColumnProfileView.deserialize(x))
        )
        return df.head(1).assign(**{_COL_PROFILE_FIELD: merged_profile.serialize()})

    def merge_to_view(self, col_profiles: List[Dict[str, Any]]) -> Iterable[Dict[str, Any]]:
        profile_view = DatasetProfileView(
            columns={
                row[_COL_NAME_FIELD]: ColumnProfileView.from_bytes(row[_COL_PROFILE_FIELD]) for row in col_profiles
            },
            dataset_timestamp=self._dataset_timestamp,
            creation_timestamp=self._creation_timestamp,
        )
        yield {self._profile_field: profile_view.serialize()}

    def profile_partition(self, df: pd.DataFrame) -> pd.DataFrame:
        if len(self._cols_to_orig) > 0:
            pdf = df.rename(columns=self._cols_to_orig)
        else:
            pdf = df
        res = why.log(pdf[self._cols] if self._cols is not None else pdf).view().serialize()
        return df.head(1)[self._by].assign(**{self._profile_field: res})  # type: ignore

    # ---------------- Starting Fugue related logic

    def transform_no_logical_partition(self, df: Any, engine: Any, engine_conf: Any) -> DatasetProfileView:
        dag = FugueWorkflow()
        input_df = dag.load(df) if isinstance(df, str) else dag.df(df)
        cols = input_df.partition(self._partition).transform(
            self.to_col_profiles,
            schema=[(_COL_NAME_FIELD, str), (_COL_PROFILE_FIELD, bytes)],
        )
        merged_cols = cols.partition_by(_COL_NAME_FIELD).transform(self.merge_col_profiles, schema="*")
        result = merged_cols.process(self.merge_to_view, schema=self._profile_schema)
        result.yield_dataframe_as("result", as_local=True)
        engine = make_execution_engine(engine, engine_conf)
        return DatasetProfileView.deserialize(dag.run(engine)["result"].as_array()[0][0])

    def transform_with_logical_partition(self, df: Any, engine: Any, engine_conf: Any, **kwargs: Any) -> Any:
        res = transform(
            df,
            transformer(lambda pdf: pdf.schema.extract(self._by) + self._profile_schema)(self.profile_partition),
            partition=self._partition,
            engine=engine,
            engine_conf=engine_conf,
            **kwargs,
        )
        return rename_dataframe_column_names(res, self._cols_to_orig)



import logging
import os
import uuid
from datetime import datetime, timezone
from functools import reduce
from glob import glob
from typing import List, Optional, Union

import whylogs as why
from whylogs.api.store.profile_store import ProfileStore
from whylogs.api.store.query import DateQuery, ProfileNameQuery
from whylogs.api.writer.local import LocalWriter
from whylogs.core import DatasetProfileView

DEFAULT_DIR = "profile_store/"
logger = logging.getLogger(__name__)


class LocalStore(ProfileStore):
    """
    The LocalStore is the implementation of the base :class:ProfileStore
    that will manage reading and writing profiles on the local file system.

    In order to instantiate the object, you will need to define a profile_name,
    which is related to the name of the model or dataset you're profiling.
    To properly use the LocalStore to generate files, you should append it
    to your existing Rolling Logger, as the below example demonstrates.

    ```python
    import whylogs as why
    from whylogs.api.store.local import LocalStore

    logger = why.logger(mode="rolling", interval=10, when="S", base_name="my_model")
    logger.append_store(store=LocalStore())

    new_df = model.predict(input_data)
    logger.log(new_df)
    ```

    The above piece of code will make sure to write the logged profiles
    to the correct location that can be further fetched using the same
    LocalStore, like so:

    ```python
    from datetime import datetime, timedelta

    from whylogs.api.store.date_config import DateConfig

    store = LocalStore()
    query = DateQuery(
        profile_name="my_model",
        start_date = datetime.now(timezone.utc) - timedelta(days=7),
        end_date = datetime.now(timezone.utc)
    )

    profile_view = store.get(query=query)
    ```

    This will fetch all existing profiles from `my_model` from the past
    7 days in a single merged DatasetProfileView.

    To list existing profiles on your local ProfileStore, you can do:

    ```python
    store = LocalStore()
    store.list()
    ```

    >**NOTE**: The parameter `profile_name` should always use the snake_case pattern,
    and it must also be **unique** to your existing dataset/ML model.
    If you use the same `profile_name` to store different profiles,
    you will end up mixing those profiles and not being able to fetch and get
    them properly again.
    """

    def __init__(self):
        self._default_path = os.path.join(os.getcwd(), DEFAULT_DIR)
        logger.debug(f"Default ProfileStore path set to {self._default_path}")
        self._writer = LocalWriter(base_dir=self._default_path)
        if not os.path.isdir(self._default_path):
            os.makedirs(os.path.join(self._default_path))

    @staticmethod
    def _get_profile_filename() -> str:
        now = datetime.now(timezone.utc)
        return f"profile_{now.date()}_{now.hour}:{now.minute}:{now.second}_{uuid.uuid4()}.bin"

    def list(self) -> List[str]:
        return [listed_file.name for listed_file in os.scandir(self._default_path) if listed_file.is_dir()]

    def get(self, query: Union[DateQuery, ProfileNameQuery]) -> Optional[DatasetProfileView]:
        logger.debug("Fetching profiles with specified StoreQuery...")
        files_list = []
        base_directory = os.path.join(self._default_path, query.profile_name)
        if isinstance(query, DateQuery):
            dates = self._get_dates(query=query)
            for date in dates:
                files_list.extend(glob(f"{base_directory}/profile_{date.strftime('%Y-%m-%d')}*.bin"))
        elif isinstance(query, ProfileNameQuery):
            files_list.extend(glob(f"{base_directory}/profile_*bin"))
        else:
            logger.warning("You must define a proper Query object")
            return None

        logger.debug(f"Profiles found! Number of profiles is {len(files_list)}")
        profiles_list = (why.read(file).view() for file in files_list)
        merged_profile = reduce(lambda x, y: x.merge(y), profiles_list)
        return merged_profile

    def write(self, profile_view: DatasetProfileView, profile_name: str) -> None:
        if not os.path.isdir(os.path.join(self._default_path, profile_name)):
            logger.debug(f"Creating directory for {profile_name}")
            os.makedirs(os.path.join(self._default_path, profile_name))

        profile_dest = os.path.join(self._default_path, profile_name, self._get_profile_filename())
        logger.debug("Writing to profile to default store destination...")
        self._writer.write(file=profile_view, dest=profile_dest)



from dataclasses import dataclass, field
from typing import List

import numpy as np


@dataclass
class HistogramConfig:
    max_hist_buckets: int = 30
    hist_avg_number_per_bucket: int = 4


@dataclass
class KSTestConfig:
    quantiles: List[float] = field(default_factory=lambda: list(np.linspace(0, 1, 100)))


@dataclass
class HellingerConfig:
    max_hist_buckets: int = 30
    hist_avg_number_per_bucket: int = 4



from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional


@dataclass
class DateQuery:
    profile_name: str
    start_date: datetime
    end_date: Optional[datetime] = field(default=None)

    def __post_init__(self):
        if self.end_date is None:
            self.end_date = self.start_date
        if self.end_date < self.start_date:
            self.start_date, self.end_date = self.end_date, self.start_date


@dataclass
class ProfileNameQuery:
    profile_name: str



import logging
from abc import ABC, abstractmethod
from copy import deepcopy
from dataclasses import dataclass, field
from itertools import chain
from typing import Any, Dict, List, Optional, Set, Union

import whylogs as why
from whylogs.api.logger.result_set import ResultSet
from whylogs.core.datatypes import (
    DataType,
    Fractional,
    Integral,
    StandardTypeMapper,
    String,
    TypeMapper,
)
from whylogs.core.metrics import StandardMetric
from whylogs.core.metrics.metrics import Metric, MetricConfig, OperationResult
from whylogs.core.metrics.multimetric import MultiMetric
from whylogs.core.preprocessing import PreprocessedColumn
from whylogs.core.resolvers import Resolver
from whylogs.core.schema import ColumnSchema, DatasetSchema

logger = logging.getLogger(__name__)

try:
    from PIL.Image import Image as ImageType
    from PIL.ImageStat import Stat
    from PIL.TiffImagePlugin import IFDRational
    from PIL.TiffTags import TAGS
except ImportError as e:
    ImageType = None
    logger.warning(str(e))
    logger.warning("Unable to load PIL; install Pillow for image support")

DEFAULT_IMAGE_FEATURES: List[str] = []


_DEFAULT_TAGS_ATTRIBUTES = [
    "ImagePixelWidth",
    "ImagePixelHeight",
    "Colorspace",
]
_IMAGE_HSV_CHANNELS = ["Hue", "Saturation", "Brightness"]
_STATS_PROPERTIES = ["mean", "stddev"]
_DEFAULT_STAT_ATTRIBUTES = [c + "." + s for c in _IMAGE_HSV_CHANNELS for s in _STATS_PROPERTIES]
_METADATA_DEFAULT_ATTRIBUTES = _DEFAULT_STAT_ATTRIBUTES + _DEFAULT_TAGS_ATTRIBUTES


def get_pil_image_statistics(
    img: ImageType, channels: List[str] = _IMAGE_HSV_CHANNELS, image_stats: List[str] = _STATS_PROPERTIES
) -> Dict:
    """
    Compute statistics data for a PIL Image

    Args:
        img (ImageType): PIL Image

    Returns:
        Dict: of metadata
    """

    stats = Stat(img.convert("HSV"))
    metadata = {}
    for index in range(len(channels)):
        for statistic_name in image_stats:
            if hasattr(stats, statistic_name):
                metadata[channels[index] + "." + statistic_name] = getattr(stats, statistic_name)[index]

    return metadata


def get_pil_exif_metadata(img: ImageType) -> Dict:
    """
    Grab EXIF metadata from image

    Args:
        img (ImageType): PIL Image

    Returns:
        Dict: of metadata
    """
    metadata = {}
    for k, v in dict(img.getexif()).items():
        try:
            if isinstance(v, IFDRational):
                metadata[TAGS[k]] = "{}".format(v)
            else:
                metadata[TAGS[k]] = v
        except KeyError:
            logger.warning(f"Couldn't read exif tag: {k} skipping.")

    return metadata


def image_based_metadata(img):
    return {
        "ImagePixelWidth": img.width,
        "ImagePixelHeight": img.height,
        "Colorspace": img.mode,
    }


def get_pil_image_metadata(img: ImageType) -> Dict:
    """
    Grab statistics data from a PIL ImageStats.Stat

    Args:
        img (ImageType): PIL Image

    Returns:
        Dict: of metadata
    """
    metadata = {}
    metadata.update(image_based_metadata(img))
    metadata.update(get_pil_image_statistics(img))
    return metadata


class SubmetricSchema(ABC):
    @abstractmethod
    def resolve(self, name: str, why_type: DataType, fi_disabled: bool = False) -> Dict[str, Metric]:
        raise NotImplementedError


class ImageSubmetricSchema(SubmetricSchema):
    def resolve(self, name: str, why_type: DataType, fi_disabled: bool = False) -> Dict[str, Metric]:
        metrics: Dict[str, Metric] = {
            "counts": StandardMetric.counts.zero(MetricConfig()),
            "types": StandardMetric.types.zero(MetricConfig()),
            "cardinality": StandardMetric.cardinality.zero(MetricConfig()),
        }

        if isinstance(why_type, Integral):
            metrics["distribution"] = StandardMetric.distribution.zero(MetricConfig())
            metrics["ints"] = StandardMetric.ints.zero(MetricConfig())
            if not fi_disabled:
                metrics["frequent_items"] = StandardMetric.frequent_items.zero(MetricConfig())
        elif isinstance(why_type, Fractional):
            metrics["distribution"] = StandardMetric.distribution.zero(MetricConfig())
        elif isinstance(why_type, String) and not fi_disabled:
            metrics["frequent_items"] = StandardMetric.frequent_items.zero(MetricConfig())

        return metrics


@dataclass(frozen=True)
class ImageMetricConfig(MetricConfig):
    allowed_exif_tags: Set[str] = field(default_factory=set)
    forbidden_exif_tags: Set[str] = field(default_factory=set)


class ImageMetric(MultiMetric):
    def __init__(
        self,
        submetrics: Dict[str, Dict[str, Metric]],
        allowed_exif_tags: Optional[Set[str]] = None,
        forbidden_exif_tags: Optional[Set[str]] = None,
        submetric_schema: Optional[SubmetricSchema] = None,
        type_mapper: Optional[TypeMapper] = None,
        fi_disabled: bool = False,
    ):
        if ImageType is None:
            logger.error("Install Pillow for image support")
        super(ImageMetric, self).__init__(submetrics)
        self._allowed_exif_tags = allowed_exif_tags or set()
        self._forbidden_exif_tags = forbidden_exif_tags or set()
        self._submetric_schema = submetric_schema or ImageSubmetricSchema()
        self._type_mapper = type_mapper or StandardTypeMapper()
        self._fi_disabled = fi_disabled

    @property
    def namespace(self) -> str:
        return "image"

    def merge(self, other: "ImageMetric") -> "ImageMetric":
        merged = super(ImageMetric, self).merge(other)
        merged._allowed_exif_tags = self._allowed_exif_tags.copy()
        merged._forbidden_exif_tags = self._forbidden_exif_tags.copy()
        merged._submetric_schema = deepcopy(self._submetric_schema)
        merged._type_mapper = deepcopy(self._type_mapper)
        merged._fi_disabled = self._fi_disabled
        return merged

    def _wants_to_track(self, exif_tag: str) -> bool:
        if exif_tag in self._forbidden_exif_tags:
            return False
        if not self._allowed_exif_tags:  # empty set -> allow anything not explicitly forbidden
            return True
        return exif_tag in self._allowed_exif_tags

    def _add_submetric(self, name: str, value: Any) -> None:
        self.submetrics[name] = self._submetric_schema.resolve(name, self._type_mapper(type(value)), self._fi_disabled)

    def _discover_exif_submetrics(self, name: str, value: Any) -> None:
        if name not in self.submetrics and self._wants_to_track(name):
            self._add_submetric(name, value)

    def _discover_image_submetrics(self, name: str, value: Any) -> None:
        if name not in self.submetrics:
            self._add_submetric(name, value)

    def _update_relevant_submetrics(self, name: str, data: PreprocessedColumn) -> None:
        if name not in self.submetrics:
            return

        for metric in self.submetrics[name].values():
            metric.columnar_update(data)

    def columnar_update(self, view: PreprocessedColumn) -> OperationResult:
        count = 0
        for image in list(chain.from_iterable(view.raw_iterator())):
            if isinstance(image, ImageType):
                metadata = get_pil_exif_metadata(image)
                for name, value in metadata.items():
                    self._discover_exif_submetrics(name, value)  # EXIF tag discovery
                    data = PreprocessedColumn.apply([metadata[name]])
                    self._update_relevant_submetrics(name, data)

                image_data = get_pil_image_metadata(image)
                for name, value in image_data.items():
                    self._discover_image_submetrics(name, value)
                    data = PreprocessedColumn.apply([image_data[name]])
                    self._update_relevant_submetrics(name, data)

                count += 1
        return OperationResult.ok(count)

    @classmethod
    def zero(cls, config: Optional[MetricConfig] = None) -> "ImageMetric":
        config = config or ImageMetricConfig()
        if not isinstance(config, ImageMetricConfig):
            logger.error("ImageMetric.zero() needs an ImageMetricConfig")
            config = ImageMetricConfig()

        return ImageMetric(
            {},  # submetrics will be discovered as images are logged
            config.allowed_exif_tags,
            config.forbidden_exif_tags,
            None,  # use standard ImageSubmetricSchema
            None,  # use standard TypeMapper
            config.fi_disabled,
        )


def log_image(
    images: Union[ImageType, List[ImageType], Dict[str, ImageType]],
    default_column_prefix: str = "image",
    schema: Optional[DatasetSchema] = None,
) -> ResultSet:
    if isinstance(images, ImageType):
        images = {default_column_prefix: images}
    if isinstance(images, list):
        count = 0
        row = dict()
        for img in images:
            row[f"{default_column_prefix}_{count}"] = img
            count += 1
        images = row

    if not isinstance(images, dict):
        raise ValueError("log_image must be passed an image, list of images, or dictionary of images")
    for img in images.values():
        if not isinstance(img, ImageType):
            raise ValueError("log_image must be passed an image, list of images, or dictionary of images")

    class ImageResolver(Resolver):
        def resolve(self, name: str, why_type: DataType, column_schema: ColumnSchema) -> Dict[str, Metric]:
            return {ImageMetric.get_namespace(): ImageMetric.zero(column_schema.cfg)}

    schema = schema or DatasetSchema(
        types={key: ImageType for key in images.keys()}, default_configs=ImageMetricConfig(), resolvers=ImageResolver()
    )
    if not isinstance(schema.default_configs, ImageMetricConfig):
        raise ValueError("log_image requires DatasetSchema with an ImageMetricConfig as default_configs")

    return why.log(row=images, schema=schema)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.wrappers_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _ModelType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ModelTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ModelType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _ModelType.ValueType  # 0
    CLASSIFICATION: _ModelType.ValueType  # 1
    REGRESSION: _ModelType.ValueType  # 2
    EMBEDDINGS: _ModelType.ValueType  # 3
    NLP: _ModelType.ValueType  # 4

class ModelType(_ModelType, metaclass=_ModelTypeEnumTypeWrapper): ...

UNKNOWN: ModelType.ValueType  # 0
CLASSIFICATION: ModelType.ValueType  # 1
REGRESSION: ModelType.ValueType  # 2
EMBEDDINGS: ModelType.ValueType  # 3
NLP: ModelType.ValueType  # 4
global___ModelType = ModelType

class CountersV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COUNT_FIELD_NUMBER: builtins.int
    TRUE_COUNT_FIELD_NUMBER: builtins.int
    NULL_COUNT_FIELD_NUMBER: builtins.int
    count: builtins.int
    @property
    def true_count(self) -> google.protobuf.wrappers_pb2.Int64Value: ...
    @property
    def null_count(self) -> google.protobuf.wrappers_pb2.Int64Value: ...
    def __init__(
        self,
        *,
        count: builtins.int = ...,
        true_count: google.protobuf.wrappers_pb2.Int64Value | None = ...,
        null_count: google.protobuf.wrappers_pb2.Int64Value | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["null_count", b"null_count", "true_count", b"true_count"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["count", b"count", "null_count", b"null_count", "true_count", b"true_count"]) -> None: ...

global___CountersV0 = CountersV0

class InferredType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Type:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[InferredType._Type.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: InferredType._Type.ValueType  # 0
        NULL: InferredType._Type.ValueType  # 1
        FRACTIONAL: InferredType._Type.ValueType  # 2
        INTEGRAL: InferredType._Type.ValueType  # 3
        BOOLEAN: InferredType._Type.ValueType  # 4
        STRING: InferredType._Type.ValueType  # 5

    class Type(_Type, metaclass=_TypeEnumTypeWrapper): ...
    UNKNOWN: InferredType.Type.ValueType  # 0
    NULL: InferredType.Type.ValueType  # 1
    FRACTIONAL: InferredType.Type.ValueType  # 2
    INTEGRAL: InferredType.Type.ValueType  # 3
    BOOLEAN: InferredType.Type.ValueType  # 4
    STRING: InferredType.Type.ValueType  # 5

    TYPE_FIELD_NUMBER: builtins.int
    RATIO_FIELD_NUMBER: builtins.int
    type: global___InferredType.Type.ValueType
    ratio: builtins.float
    def __init__(
        self,
        *,
        type: global___InferredType.Type.ValueType = ...,
        ratio: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ratio", b"ratio", "type", b"type"]) -> None: ...

global___InferredType = InferredType

class DoublesMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COUNT_FIELD_NUMBER: builtins.int
    MIN_FIELD_NUMBER: builtins.int
    MAX_FIELD_NUMBER: builtins.int
    SUM_FIELD_NUMBER: builtins.int
    count: builtins.int
    min: builtins.float
    max: builtins.float
    sum: builtins.float
    def __init__(
        self,
        *,
        count: builtins.int = ...,
        min: builtins.float = ...,
        max: builtins.float = ...,
        sum: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count", b"count", "max", b"max", "min", b"min", "sum", b"sum"]) -> None: ...

global___DoublesMessage = DoublesMessage

class LongsMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COUNT_FIELD_NUMBER: builtins.int
    MIN_FIELD_NUMBER: builtins.int
    MAX_FIELD_NUMBER: builtins.int
    SUM_FIELD_NUMBER: builtins.int
    count: builtins.int
    min: builtins.int
    max: builtins.int
    sum: builtins.int
    def __init__(
        self,
        *,
        count: builtins.int = ...,
        min: builtins.int = ...,
        max: builtins.int = ...,
        sum: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count", b"count", "max", b"max", "min", b"min", "sum", b"sum"]) -> None: ...

global___LongsMessage = LongsMessage

class VarianceMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COUNT_FIELD_NUMBER: builtins.int
    SUM_FIELD_NUMBER: builtins.int
    MEAN_FIELD_NUMBER: builtins.int
    count: builtins.int
    sum: builtins.float
    """sample variance * (n-1)"""
    mean: builtins.float
    def __init__(
        self,
        *,
        count: builtins.int = ...,
        sum: builtins.float = ...,
        mean: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count", b"count", "mean", b"mean", "sum", b"sum"]) -> None: ...

global___VarianceMessage = VarianceMessage

class FrequentNumbersSketchMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SKETCH_FIELD_NUMBER: builtins.int
    LG_MAX_K_FIELD_NUMBER: builtins.int
    sketch: builtins.bytes
    lg_max_k: builtins.int
    def __init__(
        self,
        *,
        sketch: builtins.bytes = ...,
        lg_max_k: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lg_max_k", b"lg_max_k", "sketch", b"sketch"]) -> None: ...

global___FrequentNumbersSketchMessage = FrequentNumbersSketchMessage

class FrequentItemsSketchMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SKETCH_FIELD_NUMBER: builtins.int
    LG_MAX_K_FIELD_NUMBER: builtins.int
    sketch: builtins.bytes
    lg_max_k: builtins.int
    def __init__(
        self,
        *,
        sketch: builtins.bytes = ...,
        lg_max_k: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lg_max_k", b"lg_max_k", "sketch", b"sketch"]) -> None: ...

global___FrequentItemsSketchMessageV0 = FrequentItemsSketchMessageV0

class NumbersMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VARIANCE_FIELD_NUMBER: builtins.int
    DOUBLES_FIELD_NUMBER: builtins.int
    LONGS_FIELD_NUMBER: builtins.int
    HISTOGRAM_FIELD_NUMBER: builtins.int
    THETA_FIELD_NUMBER: builtins.int
    COMPACT_THETA_FIELD_NUMBER: builtins.int
    FREQUENT_NUMBERS_FIELD_NUMBER: builtins.int
    @property
    def variance(self) -> global___VarianceMessage: ...
    @property
    def doubles(self) -> global___DoublesMessage: ...
    @property
    def longs(self) -> global___LongsMessage: ...
    histogram: builtins.bytes
    """sketches"""
    theta: builtins.bytes
    compact_theta: builtins.bytes
    @property
    def frequent_numbers(self) -> global___FrequentNumbersSketchMessage: ...
    def __init__(
        self,
        *,
        variance: global___VarianceMessage | None = ...,
        doubles: global___DoublesMessage | None = ...,
        longs: global___LongsMessage | None = ...,
        histogram: builtins.bytes = ...,
        theta: builtins.bytes = ...,
        compact_theta: builtins.bytes = ...,
        frequent_numbers: global___FrequentNumbersSketchMessage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["doubles", b"doubles", "frequent_numbers", b"frequent_numbers", "longs", b"longs", "numbers", b"numbers", "variance", b"variance"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["compact_theta", b"compact_theta", "doubles", b"doubles", "frequent_numbers", b"frequent_numbers", "histogram", b"histogram", "longs", b"longs", "numbers", b"numbers", "theta", b"theta", "variance", b"variance"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["numbers", b"numbers"]) -> typing_extensions.Literal["doubles", "longs"] | None: ...

global___NumbersMessageV0 = NumbersMessageV0

class CharPosMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class CharPosMapEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___NumbersMessageV0: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___NumbersMessageV0 | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    CHAR_LIST_FIELD_NUMBER: builtins.int
    CHAR_POS_MAP_FIELD_NUMBER: builtins.int
    char_list: builtins.str
    @property
    def char_pos_map(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___NumbersMessageV0]: ...
    def __init__(
        self,
        *,
        char_list: builtins.str = ...,
        char_pos_map: collections.abc.Mapping[builtins.str, global___NumbersMessageV0] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["char_list", b"char_list", "char_pos_map", b"char_pos_map"]) -> None: ...

global___CharPosMessage = CharPosMessage

class StringsMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COUNT_FIELD_NUMBER: builtins.int
    THETA_FIELD_NUMBER: builtins.int
    ITEMS_FIELD_NUMBER: builtins.int
    COMPACT_THETA_FIELD_NUMBER: builtins.int
    LENGTH_FIELD_NUMBER: builtins.int
    TOKEN_LENGTH_FIELD_NUMBER: builtins.int
    CHAR_POS_TRACKER_FIELD_NUMBER: builtins.int
    count: builtins.int
    theta: builtins.bytes
    """sketches"""
    items: builtins.bytes
    compact_theta: builtins.bytes
    @property
    def length(self) -> global___NumbersMessageV0: ...
    @property
    def token_length(self) -> global___NumbersMessageV0: ...
    @property
    def char_pos_tracker(self) -> global___CharPosMessage: ...
    def __init__(
        self,
        *,
        count: builtins.int = ...,
        theta: builtins.bytes = ...,
        items: builtins.bytes = ...,
        compact_theta: builtins.bytes = ...,
        length: global___NumbersMessageV0 | None = ...,
        token_length: global___NumbersMessageV0 | None = ...,
        char_pos_tracker: global___CharPosMessage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["char_pos_tracker", b"char_pos_tracker", "length", b"length", "token_length", b"token_length"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["char_pos_tracker", b"char_pos_tracker", "compact_theta", b"compact_theta", "count", b"count", "items", b"items", "length", b"length", "theta", b"theta", "token_length", b"token_length"]) -> None: ...

global___StringsMessageV0 = StringsMessageV0

class SchemaMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TypeCountsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        value: builtins.int
        def __init__(
            self,
            *,
            key: builtins.int = ...,
            value: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    TYPECOUNTS_FIELD_NUMBER: builtins.int
    INFERRED_TYPE_FIELD_NUMBER: builtins.int
    @property
    def typeCounts(self) -> google.protobuf.internal.containers.ScalarMap[builtins.int, builtins.int]: ...
    @property
    def inferred_type(self) -> global___InferredType: ...
    def __init__(
        self,
        *,
        typeCounts: collections.abc.Mapping[builtins.int, builtins.int] | None = ...,
        inferred_type: global___InferredType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["inferred_type", b"inferred_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["inferred_type", b"inferred_type", "typeCounts", b"typeCounts"]) -> None: ...

global___SchemaMessageV0 = SchemaMessageV0

class ColumnMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    COUNTERS_FIELD_NUMBER: builtins.int
    SCHEMA_FIELD_NUMBER: builtins.int
    NUMBERS_FIELD_NUMBER: builtins.int
    STRINGS_FIELD_NUMBER: builtins.int
    INFERRED_TYPE_FIELD_NUMBER: builtins.int
    FREQUENT_ITEMS_FIELD_NUMBER: builtins.int
    CARDINALITY_TRACKER_FIELD_NUMBER: builtins.int
    name: builtins.str
    @property
    def counters(self) -> global___CountersV0: ...
    @property
    def schema(self) -> global___SchemaMessageV0: ...
    @property
    def numbers(self) -> global___NumbersMessageV0: ...
    @property
    def strings(self) -> global___StringsMessageV0: ...
    @property
    def inferred_type(self) -> global___InferredType: ...
    @property
    def frequent_items(self) -> global___FrequentItemsSketchMessageV0: ...
    @property
    def cardinality_tracker(self) -> global___HllSketchMessageV0: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        counters: global___CountersV0 | None = ...,
        schema: global___SchemaMessageV0 | None = ...,
        numbers: global___NumbersMessageV0 | None = ...,
        strings: global___StringsMessageV0 | None = ...,
        inferred_type: global___InferredType | None = ...,
        frequent_items: global___FrequentItemsSketchMessageV0 | None = ...,
        cardinality_tracker: global___HllSketchMessageV0 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["cardinality_tracker", b"cardinality_tracker", "counters", b"counters", "frequent_items", b"frequent_items", "inferred_type", b"inferred_type", "numbers", b"numbers", "schema", b"schema", "strings", b"strings"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["cardinality_tracker", b"cardinality_tracker", "counters", b"counters", "frequent_items", b"frequent_items", "inferred_type", b"inferred_type", "name", b"name", "numbers", b"numbers", "schema", b"schema", "strings", b"strings"]) -> None: ...

global___ColumnMessageV0 = ColumnMessageV0

class DatasetPropertiesV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    SCHEMA_MAJOR_VERSION_FIELD_NUMBER: builtins.int
    SCHEMA_MINOR_VERSION_FIELD_NUMBER: builtins.int
    SESSION_ID_FIELD_NUMBER: builtins.int
    SESSION_TIMESTAMP_FIELD_NUMBER: builtins.int
    DATA_TIMESTAMP_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    schema_major_version: builtins.int
    schema_minor_version: builtins.int
    session_id: builtins.str
    session_timestamp: builtins.int
    data_timestamp: builtins.int
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """TODO: store other configuration here"""
    def __init__(
        self,
        *,
        schema_major_version: builtins.int = ...,
        schema_minor_version: builtins.int = ...,
        session_id: builtins.str = ...,
        session_timestamp: builtins.int = ...,
        data_timestamp: builtins.int = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_timestamp", b"data_timestamp", "metadata", b"metadata", "schema_major_version", b"schema_major_version", "schema_minor_version", b"schema_minor_version", "session_id", b"session_id", "session_timestamp", b"session_timestamp", "tags", b"tags"]) -> None: ...

global___DatasetPropertiesV0 = DatasetPropertiesV0

class ScoreMatrixMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LABELS_FIELD_NUMBER: builtins.int
    PREDICTION_FIELD_FIELD_NUMBER: builtins.int
    TARGET_FIELD_FIELD_NUMBER: builtins.int
    SCORE_FIELD_FIELD_NUMBER: builtins.int
    SCORES_FIELD_NUMBER: builtins.int
    @property
    def labels(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    prediction_field: builtins.str
    target_field: builtins.str
    score_field: builtins.str
    @property
    def scores(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NumbersMessageV0]:
        """a flattened NxN matrix (N = len(labels))"""
    def __init__(
        self,
        *,
        labels: collections.abc.Iterable[builtins.str] | None = ...,
        prediction_field: builtins.str = ...,
        target_field: builtins.str = ...,
        score_field: builtins.str = ...,
        scores: collections.abc.Iterable[global___NumbersMessageV0] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["labels", b"labels", "prediction_field", b"prediction_field", "score_field", b"score_field", "scores", b"scores", "target_field", b"target_field"]) -> None: ...

global___ScoreMatrixMessage = ScoreMatrixMessage

class RegressionMetricsMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PREDICTION_FIELD_FIELD_NUMBER: builtins.int
    TARGET_FIELD_FIELD_NUMBER: builtins.int
    COUNT_FIELD_NUMBER: builtins.int
    SUM_ABS_DIFF_FIELD_NUMBER: builtins.int
    SUM_DIFF_FIELD_NUMBER: builtins.int
    SUM2_DIFF_FIELD_NUMBER: builtins.int
    prediction_field: builtins.str
    target_field: builtins.str
    count: builtins.int
    sum_abs_diff: builtins.float
    sum_diff: builtins.float
    sum2_diff: builtins.float
    def __init__(
        self,
        *,
        prediction_field: builtins.str = ...,
        target_field: builtins.str = ...,
        count: builtins.int = ...,
        sum_abs_diff: builtins.float = ...,
        sum_diff: builtins.float = ...,
        sum2_diff: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count", b"count", "prediction_field", b"prediction_field", "sum2_diff", b"sum2_diff", "sum_abs_diff", b"sum_abs_diff", "sum_diff", b"sum_diff", "target_field", b"target_field"]) -> None: ...

global___RegressionMetricsMessage = RegressionMetricsMessage

class ModelMetricsMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SCOREMATRIX_FIELD_NUMBER: builtins.int
    MODELTYPE_FIELD_NUMBER: builtins.int
    REGRESSIONMETRICS_FIELD_NUMBER: builtins.int
    @property
    def scoreMatrix(self) -> global___ScoreMatrixMessage: ...
    modelType: global___ModelType.ValueType
    @property
    def regressionMetrics(self) -> global___RegressionMetricsMessage: ...
    def __init__(
        self,
        *,
        scoreMatrix: global___ScoreMatrixMessage | None = ...,
        modelType: global___ModelType.ValueType = ...,
        regressionMetrics: global___RegressionMetricsMessage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["regressionMetrics", b"regressionMetrics", "scoreMatrix", b"scoreMatrix"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["modelType", b"modelType", "regressionMetrics", b"regressionMetrics", "scoreMatrix", b"scoreMatrix"]) -> None: ...

global___ModelMetricsMessage = ModelMetricsMessage

class ModelProfileMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OUTPUT_FIELDS_FIELD_NUMBER: builtins.int
    METRICS_FIELD_NUMBER: builtins.int
    @property
    def output_fields(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Reserving fields for ModelMessage"""
    @property
    def metrics(self) -> global___ModelMetricsMessage: ...
    def __init__(
        self,
        *,
        output_fields: collections.abc.Iterable[builtins.str] | None = ...,
        metrics: global___ModelMetricsMessage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["metrics", b"metrics"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["metrics", b"metrics", "output_fields", b"output_fields"]) -> None: ...

global___ModelProfileMessage = ModelProfileMessage

class DatasetProfileMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ColumnsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___ColumnMessageV0: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___ColumnMessageV0 | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROPERTIES_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    MODEPROFILE_FIELD_NUMBER: builtins.int
    @property
    def properties(self) -> global___DatasetPropertiesV0: ...
    @property
    def columns(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___ColumnMessageV0]: ...
    @property
    def modeProfile(self) -> global___ModelProfileMessage:
        """reserve other fields for dataset level data"""
    def __init__(
        self,
        *,
        properties: global___DatasetPropertiesV0 | None = ...,
        columns: collections.abc.Mapping[builtins.str, global___ColumnMessageV0] | None = ...,
        modeProfile: global___ModelProfileMessage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["modeProfile", b"modeProfile", "properties", b"properties"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns", b"columns", "modeProfile", b"modeProfile", "properties", b"properties"]) -> None: ...

global___DatasetProfileMessageV0 = DatasetProfileMessageV0

class ColumnsChunkSegment(google.protobuf.message.Message):
    """*
    The follow section is for transmission and reconstruction of the dataset
    in WhyLogs backend
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MARKER_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    marker: builtins.str
    """UUID is required to aggregate to the original message
    This should map back to the original dataset
    """
    @property
    def columns(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ColumnMessageV0]: ...
    def __init__(
        self,
        *,
        marker: builtins.str = ...,
        columns: collections.abc.Iterable[global___ColumnMessageV0] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns", b"columns", "marker", b"marker"]) -> None: ...

global___ColumnsChunkSegment = ColumnsChunkSegment

class DatasetMetadataSegment(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MARKER_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    marker: builtins.str
    @property
    def properties(self) -> global___DatasetPropertiesV0: ...
    def __init__(
        self,
        *,
        marker: builtins.str = ...,
        properties: global___DatasetPropertiesV0 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["properties", b"properties"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["marker", b"marker", "properties", b"properties"]) -> None: ...

global___DatasetMetadataSegment = DatasetMetadataSegment

class MessageSegment(google.protobuf.message.Message):
    """A segment of a dataset profile. This can be used to composed the
    original object back
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MARKER_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    marker: builtins.str
    @property
    def metadata(self) -> global___DatasetMetadataSegment: ...
    @property
    def columns(self) -> global___ColumnsChunkSegment: ...
    def __init__(
        self,
        *,
        marker: builtins.str = ...,
        metadata: global___DatasetMetadataSegment | None = ...,
        columns: global___ColumnsChunkSegment | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["columns", b"columns", "item", b"item", "metadata", b"metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns", b"columns", "item", b"item", "marker", b"marker", "metadata", b"metadata"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["item", b"item"]) -> typing_extensions.Literal["metadata", "columns"] | None: ...

global___MessageSegment = MessageSegment

class HllSketchMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SKETCH_FIELD_NUMBER: builtins.int
    LG_K_FIELD_NUMBER: builtins.int
    sketch: builtins.bytes
    lg_k: builtins.int
    def __init__(
        self,
        *,
        sketch: builtins.bytes = ...,
        lg_k: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lg_k", b"lg_k", "sketch", b"sketch"]) -> None: ...

global___HllSketchMessageV0 = HllSketchMessageV0

class KllFloatsSketchMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SKETCH_FIELD_NUMBER: builtins.int
    LG_K_FIELD_NUMBER: builtins.int
    sketch: builtins.bytes
    lg_k: builtins.int
    def __init__(
        self,
        *,
        sketch: builtins.bytes = ...,
        lg_k: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lg_k", b"lg_k", "sketch", b"sketch"]) -> None: ...

global___KllFloatsSketchMessage = KllFloatsSketchMessage
